<?php
/**
 * Created by JetBrains PhpStorm.
 * User: leonardocarvalho
 * Date: 08/08/2013
 * Time: 09:45
 * To change this template use File | Settings | File Templates.
 */

namespace Curriculo\Logic;

use Indeterminado\Entities\Comunicaparc;

class LoginLogicCurriculo {

    const CDGRUPOPER_CANDIDATO = '03';
    const CDGRUPOPER_ADMIN = '02';

    private $_emailCandidato;
    private $_nrOrganizacao;
    private $_senhaCandidato;
    private $_grupOper;
    private $_cdoperador;
    private $_operador;

    public function __construct($_emailCandidato, $nrOrg, $senhaCandidato)
    {
        $this->_emailCandidato = $_emailCandidato;
        $this->_nrOrganizacao = $nrOrg;
        $this->_senhaCandidato = $senhaCandidato;

        $this->_validaLogin();
    }

    /**
     * @return mixed
     */
    public function getNrOrganizacao()
    {
        return $this->_nrOrganizacao;
    }

    /**
     * @return mixed
     */
    public function getEmailCandidato()
    {
        return $this->_emailCandidato;
    }

    /**
     *  Valida as informacoes recebidas no login.
     *
     * @return bool
     */
    private function _validaLogin() {
        $this->_validaAcesso();
        $this->_retornaOperador($this->getEmailCandidato());
    }

    /**
     *  Retorna informações do operador
     *
     * @return bool
     */
    public static function _retornaOperador($dsemailoper) {
        $operador = \OperadorEntity::retornaOperadorNmacesso($dsemailoper);

        tk_set_env_var('NMCANDIDATO', $operador->getNmoperador());
        tk_set_env_var('CDOPERADOR', $operador->getCdoperador());
        tk_set_env_var('OPERADOR', $operador->getCdoperador());
        tk_set_env_var('NRORG', $operador->getNrorg());
        tk_set_env_var('NRORG_TRAB', $operador->getNrorgselecionada());

        self::_retornaParamOper();

    }

    /**
     *  Define parametros de operador
     *
     * @return bool
     */
    public static function _retornaParamOper() {

        $cdoperador = tk_get_env_var('CDOPERADOR');
        $nrorg = tk_get_env_var('NRORG');

        $paramoper = \GpeparamoperEntity::getEntityByWhere('CDOPERADOR = ? AND NRORG = ?', array($cdoperador,$nrorg));
        if(is_object($paramoper)){
            $dtcompoper = explode(' ', $paramoper->getDtmescompetenc());

            tk_set_env_var('DTCOMPETENCIA', $dtcompoper[0]);
            tk_set_env_var('NRORG_TRAB',  $paramoper->getNrorgtrab());
        }

    }

    /**
     * Valida operador que acessa.
     *
     * @throws LoginException
     */
    private function _validaAcesso() {
        $criptoDelphi = false;
        if ($criptoDelphi) {
            $operador = $this->_validaOperadorWebService($this->_emailCandidato, $this->_senhaCandidato);
            if (is_object($operador) !== true) {
                throw new \Exception($operador);
            }
        } else {
            $this->_validaOperador();
            if ($this->_validaAlteraSenha() === false) {
                throw new \LoginException('Senha Inválida'); // Senha Invalida.
            }
        }
    }

    /**
     * Valida operador usando web service para execucao de metodo Delphi.
     *
     * @throws Exception
     * @return mixed|OperadorEntity|string
     */
    private function _validaOperadorWebService() {
        $oper_name = str_repeat('0', 12 - strlen($this->_acesso)).$this->_acesso;

        $url = 'https://www.security.teknisa.com/comwebservice/service.asmx?wsdl';

        $opcoes = array(
            'encoding'    => 'ISO-8859-1',
            'trace'       => true,
            'exceptions'  => true,
            'compression' => SOAP_COMPRESSION_ACCEPT | SOAP_COMPRESSION_GZIP,
        );

        $argumentos = array(
            new SoapVar($oper_name, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
            new SoapVar($this->_senha, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
        );

        $parametros = array(
            'rdm'    => 'Bib_Html.rdmDBGeralHtm',
            'method' => 'ValidaConexaoHTM',
            'args'   => $argumentos,
        );
        try {
            $client = new \SoapClient($url, $opcoes);
            $retorno = $client->executeCOMMethod($parametros);
            $resultado = $retorno->executeCOMMethodResult;
        } catch (SoapFault $e) {
            throw new Exception($e->getMessage());
        }
        if ($resultado === 'SenhaInv') {
            return 'Senha Inválida'; // Senha Invalida.
        }

        if ($resultado === 'OperNCad') {
            return 'Email Não Cadastrado'; // Email nao cadastrado.
        }

        $cdoperador = substr($resultado, 0, strpos($resultado, '@'));
        return \Operador::consultaOperador($cdoperador);
    }

    /**
     * Valida se operador existe ou esta inativo.
     * @throws LoginException
     */
    private function _validaOperador() {

        $email = "/[A-Za-z0-9]+([_.-][A-Za-z0-9]+)*@[A-Za-z0-9]+([_.-][A-Za-z0-9]+)*\\.[A-Za-z0-9]{2,4}/";

        if (is_numeric($this->_emailCandidato)) {
            $this->_acesso = str_repeat('0', 12 - strlen($this->_emailCandidato)).$this->_emailCandidato;
            $operador = \Operador::consultaOperador($this->_emailCandidato);
        } else if (preg_match($email, $this->_emailCandidato)) {
            $operador = \Operador::consultaOperadorPorEmail($this->_emailCandidato);
        } else {
            $operador = \Operador::consultaOperadorNmacessoUpcase($this->_emailCandidato);
        }

        if (!is_object($operador)) {
            throw new \Exception('Email Não Cadastrado'); // Email Nao Cadastrado.
        }

        if ($operador->getIdoperativo() !== 'A'){
            throw new \Exception('Email Inativo'); // Email encontra-se inativo.
        }

        $this->_cdoperador = $operador->getCdoperador();
        $this->_operador = $operador;
        $this->_validaGrupOper();

    }

    /**
     *  Valida ou altera senha.
     *
     * @return void|boolean
     */
    private function _validaAlteraSenha(){

        $criptosenha = $this->_senhaCandidato;
        $cdsenhaoperweb = ($this->_senhaCandidato !== '')?$this->_operador->getCdsenhaoperweb() : null;

        if (($cdsenhaoperweb === null || $cdsenhaoperweb === 'd41d8cd98f00b204e9800998ecf8427e') && $this->_senhaCandidato !== '') {
            \Operador::alteraSenhaOperador($this->_operador->getCdoperador(), $criptosenha);
            return null;
        } else if ($cdsenhaoperweb !== $criptosenha) {
            return false;
        }
        return true;
    }

    /**
     * Retorna grupo operador
     */
    private function _validaGrupOper()
    {

        $this->_grupOper = \Opergrupop::retornaOperGrupOp($this->_nrOrganizacao, $this->_cdoperador);
        if (is_array($this->_grupOper)) {
            foreach($this->_grupOper as $grupo){
                if($grupo->getCdgrupoper() == self::CDGRUPOPER_CANDIDATO
                    || $grupo->getCdgrupoper() == self::CDGRUPOPER_ADMIN){
                    tk_set_env_var('GRUPOPER', $grupo->getCdgrupoper());
                }
            }
        } else {
            throw new \Exception('Operador sem grupo definido.');
        }
    }

}